import { __assign } from "tslib";
export var aggregateToQuery = function (queries) {
    var _a;
    var isLoading = queries.some(function (query) { return query.isLoading; });
    var isError = queries.some(function (query) { return query.isError; });
    var isFetching = queries.some(function (query) { return query.isFetching; });
    var isUninitialized = queries.some(function (query) { return query.isUninitialized; });
    var isSuccess = !isUninitialized && !isLoading && !isError;
    var error = (_a = queries.find(function (query) { return query.error !== undefined; })) === null || _a === void 0 ? void 0 : _a.error;
    var fulfilledTimeStamp = Math.max.apply(Math, queries
        .filter(function (query) { return typeof query.fulfilledTimeStamp === "number"; })
        .map(function (query) { return query.fulfilledTimeStamp; }));
    var startedTimeStamp = Math.max.apply(Math, queries
        .filter(function (query) { return typeof query.startedTimeStamp === "number"; })
        .map(function (query) { return query.startedTimeStamp; }));
    var requestId = queries
        .filter(function (query) { return typeof query.requestId === "string"; })
        .map(function (query) { return query.requestId; })
        .join("");
    var refetch = function () {
        queries.forEach(function (query) { return query.refetch(); });
    };
    return {
        isLoading: isLoading,
        isError: isError,
        isFetching: isFetching,
        isSuccess: isSuccess,
        isUninitialized: isUninitialized,
        refetch: refetch,
        error: error,
        fulfilledTimeStamp: fulfilledTimeStamp,
        startedTimeStamp: startedTimeStamp,
        requestId: requestId
    };
};
export var createLoader = function (createLoaderArgs) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var createdQueries = createLoaderArgs.queries.apply(createLoaderArgs, args);
        var aggregatedQuery = aggregateToQuery(createdQueries);
        if (aggregatedQuery.isSuccess) {
            var data = createLoaderArgs.transform
                ? createLoaderArgs.transform(createdQueries)
                : createdQueries;
            return __assign(__assign({}, aggregatedQuery), { data: data });
        }
        return aggregatedQuery;
    };
};
export { RTKLoader } from "./RTKLoader";
//# sourceMappingURL=index.js.map