"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLoader = exports.aggregateToQuery = void 0;
const aggregateToQuery = (queries) => {
    var _a;
    const isLoading = queries.some((query) => query.isLoading);
    const isError = queries.some((query) => query.isError);
    const isFetching = queries.some((query) => query.isFetching);
    const isUninitialized = queries.some((query) => query.isUninitialized);
    const isSuccess = !isUninitialized && !isLoading && !isError;
    const error = (_a = queries.find((query) => query.error !== undefined)) === null || _a === void 0 ? void 0 : _a.error;
    const fulfilledTimeStamp = Math.max(...queries
        .filter((query) => typeof query.fulfilledTimeStamp === "number")
        .map((query) => query.fulfilledTimeStamp));
    const startedTimeStamp = Math.max(...queries
        .filter((query) => typeof query.startedTimeStamp === "number")
        .map((query) => query.startedTimeStamp));
    const requestId = queries
        .filter((query) => typeof query.requestId === "string")
        .map((query) => query.requestId)
        .join("");
    const refetch = () => {
        queries.forEach((query) => query.refetch());
    };
    return {
        isLoading,
        isError,
        isFetching,
        isSuccess,
        isUninitialized,
        refetch,
        error,
        fulfilledTimeStamp,
        startedTimeStamp,
        requestId,
    };
};
exports.aggregateToQuery = aggregateToQuery;
const createLoader = (createLoaderArgs) => {
    return (...args) => {
        const createdQueries = createLoaderArgs.queries(...args);
        const aggregatedQuery = (0, exports.aggregateToQuery)(createdQueries);
        if (aggregatedQuery.isSuccess) {
            const data = createLoaderArgs.transform
                ? createLoaderArgs.transform(createdQueries)
                : createdQueries;
            return Object.assign(Object.assign({}, aggregatedQuery), { data });
        }
        return aggregatedQuery;
    };
};
exports.createLoader = createLoader;
